package com.caimei365.manager.service.caimei.impl;

import cn.hutool.core.lang.Assert;
import com.alibaba.excel.util.StringUtils;
import com.caimei365.manager.dao.KeyWordDao;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.caimei.CmPriorKeyword;
import com.caimei365.manager.entity.caimei.KeyWord;
import com.caimei365.manager.entity.caimei.cmUser.ServiceProviderModel;
import com.caimei365.manager.entity.caimei.enums.KeyWordEnum;
import com.caimei365.manager.service.caimei.KeyWordService;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;


import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zzj
 */
@Slf4j
@Service
public class KeyWordServiceImpl implements KeyWordService {


    @Resource
    private KeyWordDao keyWordDao;

    @Override
    public ResponseJson<PaginationVo<KeyWord>> getKeyWordList(String keyword, Integer searchTimeCode, Integer fromSearch, String beginTime, String endTime, Integer labelStatus, int pageNum, int pageSize) {
        if (null != searchTimeCode && searchTimeCode > 0 && StringUtils.isEmpty(beginTime) && StringUtils.isEmpty(endTime)) {
            Map<String, String> timeVal = KeyWordEnum.getTimeVal(searchTimeCode);
            beginTime = timeVal.get("beginTime");
            endTime = timeVal.get("endTime");
        }
        if (!StringUtils.isEmpty(beginTime)) {
            beginTime += " 00:00:00";
        }
        if (!StringUtils.isEmpty(endTime)) {
            endTime += " 23:59:59";
        }
        PageHelper.startPage(pageNum, pageSize);
        List<KeyWord> keyWords = keyWordDao.findList(keyword, fromSearch, beginTime, endTime, labelStatus);
        PaginationVo<KeyWord> pageData = new PaginationVo<>(keyWords);
        return ResponseJson.success(pageData);
    }

    @Override
    public ResponseJson joinLabel(String id) {
        Assert.notBlank(id);
        if (id.contains(",")) {
            String[] split = id.split(",");
            for (String s : split) {
                keyWordDao.updateLabelStatus(s);
            }
        } else {
            keyWordDao.updateLabelStatus(id);
        }
        return ResponseJson.success();
    }

    @Override
    public ResponseJson<PaginationVo<KeyWord>> getRecommendList(String keyword, String beginTime, String endTime, Integer pageNum, Integer pageSize) {
        if (!StringUtils.isEmpty(beginTime)) {
            beginTime += " 00:00:00";
        }
        if (!StringUtils.isEmpty(endTime)) {
            endTime += " 23:59:59";
        }
        PageHelper.startPage(pageNum, pageSize);
        List<KeyWord> keyWords = keyWordDao.findRecommendList(keyword, beginTime, endTime);
        PaginationVo<KeyWord> pageData = new PaginationVo<>(keyWords);
        return ResponseJson.success(pageData);
    }

    @Override
    public ResponseJson ignoreLabel(String id) {
        Assert.notBlank(id);
        if (id.contains(",")) {
            String[] split = id.split(",");
            for (String s : split) {
                keyWordDao.updateRecommendFlag(s);
            }
        } else {
            keyWordDao.updateRecommendFlag(id);
        }
        return ResponseJson.success();
    }

    @Override
    public ResponseJson insertLabel(String keyword) {
        Integer keywordExist = keyWordDao.findKeywordExist(keyword);
        if (null == keywordExist) {
            keyWordDao.insertLabel(keyword);
            return ResponseJson.success();
        } else {
            return ResponseJson.success("关键词已存在");
        }
    }

    @Override
    public ResponseJson deleteLabel(String id) {
        Assert.notBlank(id);
        if (id.contains(",")) {
            String[] split = id.split(",");
            for (String s : split) {
                keyWordDao.updateDelFlag(s);
                keyWordDao.delPriorKeywordById(Integer.parseInt(s));
            }
        } else {
            keyWordDao.updateDelFlag(id);
        }
        return ResponseJson.success();
    }

    /**
     * 协销列表
     *
     * @return
     */
    @Override
    public ResponseJson<List<ServiceProviderModel>> getServiceList() {
        return ResponseJson.success(keyWordDao.getServiceList());
    }

    /**
     * 静态标签列表
     * @param keyword
     * @param fromSearch
     * @param beginTime
     * @param endTime
     * @param serviceProviderId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<KeyWord>> getStateKeyWordList(String keyword, Integer fromSearch, String beginTime, String endTime, Integer serviceProviderId, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<KeyWord> keyWords = keyWordDao.getStateKeyWordList(keyword, fromSearch, beginTime, endTime, serviceProviderId);
        PaginationVo<KeyWord> pageData = new PaginationVo<>(keyWords);
        return ResponseJson.success(pageData);
    }

    /**
     * 插入静态标签
     *
     * @param keyword
     * @return
     */
    @Override
    public ResponseJson insertLabelByState(String keyword) {
        Integer keywordExist = keyWordDao.findKeywordExist(keyword);
        if (null == keywordExist) {
            keyWordDao.insertLabelByState(keyword);
            return ResponseJson.success();
        } else {
            return ResponseJson.success("关键词已存在");
        }
    }

    /**
     * 优先展示标签列表
     *
     * @param keyword
     * @param fromSearch
     * @param dynamicStatus
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmPriorKeyword>> getPriorKeywordList(String keyword, Integer fromSearch, Integer dynamicStatus, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmPriorKeyword> priorKeywordList = keyWordDao.getPriorKeywordList(keyword, fromSearch, dynamicStatus);

        PaginationVo<CmPriorKeyword> page = new PaginationVo<>(priorKeywordList);
        return ResponseJson.success(page);
    }

    /**
     * 选择标签
     *
     * @param keyword
     * @param fromSearch
     * @param dynamicStatus 0 为动态标签 1 静态标签
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<KeyWord>> getPickKeyword(String keyword, Integer fromSearch, Integer dynamicStatus, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<KeyWord> priorKeywordList = keyWordDao.getPickKeyword(keyword, fromSearch, dynamicStatus);
        List<CmPriorKeyword> keywordList = keyWordDao.getPriorKeywordList("",null,null);
        List<Integer> searchIds = new ArrayList<>();
        if (null != keywordList && keywordList.size()>0) {
            searchIds = keywordList.stream().map(CmPriorKeyword::getSearchId).collect(Collectors.toList());
        }
        for (KeyWord key : priorKeywordList) {
            if (searchIds.contains(key.getId())) {
                key.setFlag(false);
            } else {
                key.setFlag(true);
            }
        }
        PaginationVo<KeyWord> page = new PaginationVo<>(priorKeywordList);
        return ResponseJson.success(page);
    }

    /**
     * 保存优先展示标签
     *
     * @param searchIds
     * @return
     */
    @Override
    public ResponseJson savePriorKeyword(String searchIds) {
        List<Integer> searchIdList = new ArrayList<>();
        if (searchIds.contains(",")) {
            String[] split = searchIds.split(",");
            for (String s : split) {
                searchIdList.add(Integer.parseInt(s));
            }
        } else {
            searchIdList.add(Integer.parseInt(searchIds));
        }
        Integer count = keyWordDao.getPriorCount();
        if (count > 50) {
            return ResponseJson.error(-1, "只支持展示50个标签，请删减再添加。", null);
        }
        if ((count + searchIdList.size()) > 50) {
            int num = (count + searchIdList.size()) - 50;
            return ResponseJson.error(-1, "只支持展示50个标签，已超出"+ num +"(个)，请删减再添加。", null);
        }
        // 保存优先展示标签
        for (Integer id : searchIdList) {
            Integer prior = keyWordDao.getPrior(id);
            if (null == prior) {
                keyWordDao.insertPriorKeyword(id);
            } else {
                return ResponseJson.success("关键词已存在");
            }
        }
        return ResponseJson.success();
    }

    /**
     * 删除优先展示标签
     *
     * @param ids
     * @return
     */
    @Override
    public ResponseJson delPriorKeyword(String ids) {
        if (ids.contains(",")) {
            String[] split = ids.split(",");
            for (String id : split) {
                keyWordDao.delPriorKeyword(Integer.parseInt(id));
            }
        } else {
            keyWordDao.delPriorKeyword(Integer.parseInt(ids));
        }
        return ResponseJson.success();
    }
}
