package com.caimei365.manager.service.caimei.user.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.manager.dao.user.CustomerServiceDao;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.caimei.CmShop;
import com.caimei365.manager.entity.caimei.cmUser.*;
import com.caimei365.manager.service.caimei.user.CustomerService;
import com.github.pagehelper.PageHelper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Description
 *
 * @author : Charles
 * @date : 2023/9/5
 */
@Service
public class CustomerServiceImpl implements CustomerService {

    @Resource private CustomerServiceDao customerServiceDao;

    /**
     * 潜客供应商列表
     * @param cmShopStatistics
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmShopStatistics>> getCustomerShopList(CmShopStatistics cmShopStatistics, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmShopStatistics> customerShopList = customerServiceDao.getCustomerShopList(cmShopStatistics);
        PaginationVo<CmShopStatistics> page = new PaginationVo<>(customerShopList);
        return ResponseJson.success(page);
    }

    /**
     * 供应商列表
     * @param shopName
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmShop>> getShopLiat(String shopName, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmShop> shopList = customerServiceDao.getShopList(shopName, null);
        // 是否支持选择
        List<Integer> customerShopId = customerServiceDao.getCustomerShopId();
        shopList.forEach( s -> {
            if (customerShopId.contains(s.getShopId())) {
                s.setFlag(false);
            } else {
                s.setFlag(true);
            }
        });
        PaginationVo<CmShop> page = new PaginationVo<>(shopList);
        return ResponseJson.success(page);
    }

    /**
     * 添加统计供应商
     *
     * @param shopIds
     * @return
     */
    @Override
    public ResponseJson saveCustomerShop(String shopIds) {
        List<String> shopId = new ArrayList<>();
        if (shopIds.contains(",")) {
            shopId = Arrays.asList(shopIds.split(","));
        } else {
            shopId.add(shopIds);
        }
        for (String s : shopId) {
            CmShopStatistics statistics = new CmShopStatistics();
            statistics.setShopId(Integer.parseInt(s));
            statistics.setStatus(0);
            statistics.setDelFlag(0);
            customerServiceDao.insertCustomerShop(statistics);
        }
        return ResponseJson.success();
    }

    /**
     * 设置统计状态
     *
     * @param id
     * @param status
     * @return
     */
    @Override
    public ResponseJson renewCustomerShop(Integer id, Integer status) {
        customerServiceDao.updateCustomerShop(id, status);
        return ResponseJson.success();
    }

    /**
     * 游客统计
     *
     * @param cmRoosInformation
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmRoosInformation>> getCmInformationList(CmRoosInformation cmRoosInformation, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmRoosInformation> cmInformationList = customerServiceDao.getCmInformationList(cmRoosInformation);
        PaginationVo<CmRoosInformation> page = new PaginationVo<>(cmInformationList);
        return ResponseJson.success(page);
    }

    /**
     * 设置跟进状态
     *
     * @param id
     * @param followUpStatus
     * @return
     */
    @Override
    public ResponseJson renewFollowUpStatus(Integer id, Integer followUpStatus) {
        customerServiceDao.updateInformation(id, followUpStatus);
        return ResponseJson.success();
    }

    /**
     * 相关文章列表
     *
     * @param cmShopInfo
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmShopInfo>> getShopInfoList(CmShopInfo cmShopInfo, Integer pageNum, Integer pageSize) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        PageHelper.startPage(pageNum, pageSize);
        List<CmShopInfo> shopInfoList = customerServiceDao.getShopInfoList(cmShopInfo);
        for (CmShopInfo shopInfo : shopInfoList) {
            if (null != shopInfo.getPubdate()) {
                shopInfo.setPubdateString(dateFormat.format(shopInfo.getPubdate()));
            }
        }
        PaginationVo<CmShopInfo> page = new PaginationVo<>(shopInfoList);
        return ResponseJson.success(page);
    }

    /**
     * 选择文章
     *
     * @param id
     * @param title
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<Info>> getInfoList(Integer id, String title, Integer pageNum, Integer pageSize) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        PageHelper.startPage(pageNum, pageSize);
        List<Info> infoList = customerServiceDao.getInfoList(id, title);
        List<Integer> shopInfoId = customerServiceDao.getShopInfoId();
        infoList.forEach( i -> {
            if (shopInfoId.contains(i.getId())) {
                i.setFlag(false);
            } else {
                i.setFlag(true);
            }

            if (null != i.getCreateDate()) {
                i.setCreateDateString(dateFormat.format(i.getCreateDate()));
            }
            if (null != i.getPubdate()) {
                i.setPubdateString(dateFormat.format(i.getPubdate()));
            }

        });
        PaginationVo<Info> page = new PaginationVo<>(infoList);
        return ResponseJson.success(page);
    }

    /**
     * 添加文章统计
     *
     * @param shopId
     * @param infoIds
     * @return
     */
    @Override
    public ResponseJson saveShopInfo(Integer shopId, String infoIds) {
        List<String> infoId = new ArrayList<>();
        if (infoIds.contains(",")) {
            infoId = Arrays.asList(infoIds.split(","));
        } else {
            infoId.add(infoIds);
        }
        for (String s : infoId) {
            CmShopInfo shopInfo = new CmShopInfo();
            shopInfo.setShopId(shopId);
            shopInfo.setInfoId(Integer.parseInt(s));
            shopInfo.setStatus(0);
            shopInfo.setDelFlag(0);
            customerServiceDao.insertShopInfo(shopInfo);
        }
        return ResponseJson.success();
    }

    /**
     * 设置统计状态
     *
     * @param id
     * @param status
     * @return
     */
    @Override
    public ResponseJson renewShopInfo(Integer id, Integer status) {
        customerServiceDao.updateShopInfoStatus(id, status);
        return ResponseJson.success();
    }

    /**
     * 删除统计文章
     *
     * @param id
     * @return
     */
    @Override
    public ResponseJson delShopInfo(Integer id) {
        customerServiceDao.delShopInfo(id);
        return ResponseJson.success();
    }

    /**
     * 相关搜索词列表
     *
     * @param cmShopKeyword
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmShopKeyword>> getShopKeywordList(CmShopKeyword cmShopKeyword, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmShopKeyword> shopKeyword = customerServiceDao.getShopKeyword(cmShopKeyword);
        PaginationVo<CmShopKeyword> page = new PaginationVo<>(shopKeyword);
        return ResponseJson.success(page);
    }

    /**
     * 选择搜索词
     *
     * @param keyword
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmUserSearchFrequency>> getSearchFrequencyList(String keyword, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmUserSearchFrequency> searchFrequencyList = customerServiceDao.getSearchFrequencyList(keyword);
        List<Integer> shopKeywordId = customerServiceDao.getShopKeywordId();
        searchFrequencyList.forEach( s -> {
            if (shopKeywordId.contains(s.getId())) {
                s.setFlag(false);
            } else {
                s.setFlag(true);
            }
        });
        PaginationVo<CmUserSearchFrequency> page = new PaginationVo<>(searchFrequencyList);
        return ResponseJson.success(page);
    }

    /**
     * 添加搜索词统计
     *
     * @param shopId
     * @param keyWordIds
     * @return
     */
    @Override
    public ResponseJson saveShopKeyword(Integer shopId, String keyWordIds) {
        List<String> keyWordId = new ArrayList<>();
        if (keyWordIds.contains(",")) {
            keyWordId = Arrays.asList(keyWordIds.split(","));
        } else {
            keyWordId.add(keyWordIds);
        }
        for (String s : keyWordId) {
            CmShopKeyword shopKeyword = new CmShopKeyword();
            shopKeyword.setShopId(shopId);
            shopKeyword.setSearchId(Integer.parseInt(s));
            shopKeyword.setStatus(0);
            shopKeyword.setDelFlag(0);
            customerServiceDao.insertShopKeyword(shopKeyword);
        }
        return ResponseJson.success();
    }

    /**
     * 设置统计状态
     *
     * @param id
     * @param status
     * @return
     */
    @Override
    public ResponseJson renewShopKeyword(Integer id, Integer status) {
        customerServiceDao.updateShopKeywordStatus(id, status);
        return ResponseJson.success();
    }

    /**
     * 删除统计搜索词
     *
     * @param id
     * @return
     */
    @Override
    public ResponseJson delShopKeyword(Integer id) {
        customerServiceDao.delShopKeyword(id);
        return ResponseJson.success();
    }

    /**
     * 弹框数据回显
     *
     * @param shopId
     * @return
     */
    @Override
    public ResponseJson<CmShopPopUp> getShopPopUp(Integer shopId) {
        return ResponseJson.success(customerServiceDao.getShopPopUp(shopId));
    }

    /**
     * 弹框数据保存
     *
     * @param cmShopPopUp
     * @return
     */
    @Override
    public ResponseJson saveShopPopUp(CmShopPopUp cmShopPopUp) {
        if (null == cmShopPopUp.getId()) {
            // 新增
            customerServiceDao.insertShopPopUp(cmShopPopUp);
        } else {
            // 修改
            customerServiceDao.updateShopPopUp(cmShopPopUp);
        }
        return ResponseJson.success();
    }

    /**
     * 供应商广告列表
     *
     * @param cmShopAdvertisingImage
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmShopAdvertisingImage>> getShopAdvertisingImage(CmShopAdvertisingImage cmShopAdvertisingImage, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmShopAdvertisingImage> shopAdvertisingImage = customerServiceDao.getShopAdvertisingImage(cmShopAdvertisingImage);
        PaginationVo<CmShopAdvertisingImage> page = new PaginationVo<>(shopAdvertisingImage);
        return ResponseJson.success(page);
    }

    /**
     * 供应商广告信息回显
     *
     * @param id
     * @return
     */
    @Override
    public ResponseJson<CmShopAdvertisingImage> getShopAdvertisingImageById(Integer id) {
        CmShopAdvertisingImage shopAdvertisingImage = customerServiceDao.getShopAdvertisingImageById(id);
        // 设置供应商信息
        List<CmShop> shopList = customerServiceDao.getShopList("",shopAdvertisingImage.getShopId());
        List<CmShopStatistics> shops = new ArrayList<>();
        CmShop shop = shopList.stream().filter(s -> s.getShopId().equals(shopAdvertisingImage.getShopId())).collect(Collectors.toList()).get(0);
        // 回显设置
        CmShopStatistics cmShopStatistics = new CmShopStatistics();
        cmShopStatistics.setShopId(shop.getShopId());
        cmShopStatistics.setShopName(shop.getName());
        cmShopStatistics.setLinkMan(shop.getLinkMan());
        cmShopStatistics.setMobile(shop.getContractMobile());
        cmShopStatistics.setStatus(shop.getStatus());
        shops.add(cmShopStatistics);
        shopAdvertisingImage.setShops(shops);
        return ResponseJson.success(shopAdvertisingImage);
    }

    /**
     * 保存供应商广告信息
     *
     * @param cmShopAdvertisingImage
     * @return
     */
    @Override
    public ResponseJson saveShopAdvertisingImage(CmShopAdvertisingImage cmShopAdvertisingImage) {
        if (null == cmShopAdvertisingImage.getId()) {
            cmShopAdvertisingImage.setSort(0);
            cmShopAdvertisingImage.setDelFlag(0);
            // 插入
            customerServiceDao.insertShopAdvertisingImage(cmShopAdvertisingImage);
        } else {
            // 修改
            customerServiceDao.updateShopAdvertisingImage(cmShopAdvertisingImage);
        }
        return ResponseJson.success();
    }

    /**
     * 一键排序
     *
     * @param id
     * @param sort
     * @return
     */
    @Override
    public ResponseJson renewShopAdvertisingImageSort(Integer id, Integer sort) {
        customerServiceDao.updateShopAdvertisingImageSort(id, sort);
        return ResponseJson.success();
    }

    /**
     * 修改上下架状态
     *
     * @param id
     * @param status
     * @return
     */
    @Override
    public ResponseJson renewShopAdvertisingImageStatus(Integer id, Integer status) {
        customerServiceDao.updateShopAdvertisingImageStatus(id, status);
        return ResponseJson.success();
    }

    /**
     * 删除广告
     *
     * @param id
     * @return
     */
    @Override
    public ResponseJson delShopAdvertisingImage(Integer id) {
        customerServiceDao.delShopAdvertisingImage(id);
        return ResponseJson.success();
    }

    /**
     * 供应商主页统计
     *
     * @param shopId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmPageShop>> getPageShop(Integer shopId, String startTime, String endTime, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmPageShop> pageShop = customerServiceDao.getPageShop(shopId, startTime, endTime);
        pageShop.forEach(shop -> {
            shop.setAverage(calculationTime(shop.getAverageDuration().toString()));
        });
        PaginationVo<CmPageShop> page = new PaginationVo<>(pageShop);
        return ResponseJson.success(page);
    }

    /**
     * 供应商商品统计
     *
     * @param shopId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmPageShopProduct>> getPageShopProduct(Integer shopId, String startTime, String endTime, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmPageShopProduct> pageShopProduct = customerServiceDao.getPageShopProduct(shopId, startTime, endTime);
        pageShopProduct.forEach(shop -> {
            shop.setAverage(calculationTime(shop.getAverageDuration().toString()));
        });
        PaginationVo<CmPageShopProduct> page = new PaginationVo<>(pageShopProduct);
        return ResponseJson.success(page);
    }

    /**
     * 供应商文章
     *
     * @param shopId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmPageShopInfo>> getPageShopInfo(Integer shopId, String startTime, String endTime, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmPageShopInfo> pageShopInfo = customerServiceDao.getPageShopInfo(shopId, startTime, endTime);
        pageShopInfo.forEach(shop -> {
            shop.setAverage(calculationTime(shop.getAverageDuration().toString()));
        });
        PaginationVo<CmPageShopInfo> page = new PaginationVo<>(pageShopInfo);
        return ResponseJson.success(page);
    }

    /**
     * 供应商搜索词统计
     *
     * @param shopId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmPageShopKeyword>> getPageShopKeyword(Integer shopId, String startTime, String endTime, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmPageShopKeyword> pageShopKeyword = customerServiceDao.getPageShopKeyword(shopId, startTime, endTime);
        pageShopKeyword.forEach(shop -> {
            shop.setAverage(calculationTime(shop.getAverageDuration().toString()));
        });
        PaginationVo<CmPageShopKeyword> page = new PaginationVo<>(pageShopKeyword);
        return ResponseJson.success(page);
    }


    /**
     * 时间计算
     * @param recordTime
     * @return
     */
    public String calculationTime(String recordTime){
        double doc = Double.parseDouble(recordTime);
        int num = (int)doc;
        //小时
        int HH = 0;
        // 分钟
        int mm = 0;
        //秒
        int ss = 0;
        int item = num / 1000;
        if ((item / 60) >0) {
            mm = item / 60;
            ss = item % 60;
        } else {
            ss = item;
        }
        if ((mm / 60) >0) {
            HH = mm / 60;
            mm = mm % 60;
        }
        String  str= HH+":"+mm+":"+ss;
        SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
        Date parse = null;
        try {
            parse = dateFormat.parse(str);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return dateFormat.format(parse);
    }
}
